/*!
 * @file main.c
 * @brief USB UART 5 Click Example.
 *
 * # Description
 * This example reads and processes data from USB UART 5 Click board™.
 * The library initializes and defines the UART bus drivers 
 * to transmit or receive data.
 *
 * The demo application is composed of two sections :
 *
 * ## Application Init
 * Initializes driver, wake-up module, and performs the default configuration.
 *
 * ## Application Task
 * Transmitter/Receiver task depends on uncommented code. 
 * The receiver logs each received byte to the UART for data logging, 
 * while transmitted sends messages every 2 seconds.
 * Results are being sent to the UART Terminal, where you can track their changes.
 *
 * ## Additional Function
 * - static void usbuart5_clear_app_buf ( void )
 * - static err_t usbuart5_process ( void )
 *
 * @author Nenad Filipovic
 *
 */

#include "board.h"
#include "log.h"
#include "usbuart5.h"

#define DEMO_APP_RECEIVER
// #define DEMO_APP_TRANSMITTER
 
#define PROCESS_BUFFER_SIZE 200

static usbuart5_t usbuart5;
static log_t logger;

static uint8_t message_data[ 9 ] = { 'M', 'i', 'k', 'r', 'o', 'E', 13, 10, 0 };
static char app_buf[ PROCESS_BUFFER_SIZE ] = { 0 };
static int32_t app_buf_len = 0;
static int32_t app_buf_cnt = 0;

/**
 * @brief USB UART 5 clearing application buffer.
 * @details This function clears memory of application buffer and reset its length and counter.
 * @note None.
 */
static void usbuart5_clear_app_buf ( void );

/**
 * @brief USB UART 5 data reading function.
 * @details This function reads data from device and concatenates data to application buffer.
 * @return @li @c  0 - Read some data.
 *         @li @c -1 - Nothing is read.
 *         @li @c -2 - Application buffer overflow.
 * See #err_t definition for detailed explanation.
 * @note None.
 */
static err_t usbuart5_process ( void );

void application_init ( void ) 
{
    log_cfg_t log_cfg;  /**< Logger config object. */
    usbuart5_cfg_t usbuart5_cfg;  /**< Click config object. */

    /** 
     * Logger initialization.
     * Default baud rate: 115200
     * Default log level: LOG_LEVEL_DEBUG
     * @note If USB_UART_RX and USB_UART_TX 
     * are defined as HAL_PIN_NC, you will 
     * need to define them manually for log to work. 
     * See @b LOG_MAP_USB_UART macro definition for detailed explanation.
     */
    LOG_MAP_USB_UART( log_cfg );
    log_init( &logger, &log_cfg );
    log_info( &logger, " Application Init " );

    // Click initialization.
    usbuart5_cfg_setup( &usbuart5_cfg );
    USBUART5_MAP_MIKROBUS( usbuart5_cfg, MIKROBUS_1 );
    if ( UART_ERROR == usbuart5_init( &usbuart5, &usbuart5_cfg ) ) 
    {
        log_error( &logger, " Communication init." );
        for ( ; ; );
    }

    if ( USBUART5_ERROR == usbuart5_default_cfg ( &usbuart5 ) )
    {
        log_error( &logger, " Default configuration." );
        for ( ; ; );
    }

    log_info( &logger, " Application Task " );
    app_buf_len = 0;
    app_buf_cnt = 0;
#ifdef DEMO_APP_RECEIVER
    log_info( &logger, " RECEIVER " );
#endif

#ifdef DEMO_APP_TRANSMITTER
    log_info( &logger, " TRANSMITTER " );
#endif
}

void application_task ( void ) 
{
#ifdef DEMO_APP_RECEIVER
    usbuart5_process();
    if ( app_buf_len > 0 ) 
    {
        log_printf( &logger, "%s", app_buf );
        usbuart5_clear_app_buf(  );
    }
#endif

#ifdef DEMO_APP_TRANSMITTER
    if ( USBUART5_ERROR != usbuart5_send_data( &usbuart5, &message_data[ 0 ] ) )
    {
        log_printf( &logger, "MESSAGE SENT\r\n" );    
    }
    Delay_ms( 2000 );
#endif
}

void main ( void ) 
{
    application_init( );

    for ( ; ; ) 
    {
        application_task( );
    }
}

static void usbuart5_clear_app_buf ( void ) 
{
    memset( app_buf, 0, app_buf_len );
    app_buf_len = 0;
    app_buf_cnt = 0;
}

static err_t usbuart5_process ( void ) 
{
    int32_t rx_size;
    char rx_buf[ PROCESS_BUFFER_SIZE ] = { 0 };
    rx_size = usbuart5_generic_read( &usbuart5, rx_buf, PROCESS_BUFFER_SIZE );
    if ( rx_size > 0 ) 
    {
        int32_t buf_cnt = 0;
        if ( ( app_buf_len + rx_size ) > PROCESS_BUFFER_SIZE ) 
        {
            usbuart5_clear_app_buf(  );
            return USBUART5_ERROR;
        } 
        else 
        {
            buf_cnt = app_buf_len;
            app_buf_len += rx_size;
        }
        for ( int32_t rx_cnt = 0; rx_cnt < rx_size; rx_cnt++ ) 
        {
            if ( rx_buf[ rx_cnt ] != 0 ) 
            {
                app_buf[ ( buf_cnt + rx_cnt ) ] = rx_buf[ rx_cnt ];
            }
            else
            {
                app_buf_len--;
                buf_cnt--;
            }
        }
        return USBUART5_OK;
    }
    return USBUART5_ERROR;
}

// ------------------------------------------------------------------------ END
